
// Boost. Range Template Library (rtl)
//
// Copyright 2003 John Torjo (john@torjo.com) and Matthew Wilson (matthew@synesis.com.au)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.


#ifndef BOOST_RTL_REVERSE_HPP_INCLUDED
#define BOOST_RTL_REVERSE_HPP_INCLUDED

#include <boost/rtl/priv/defs.hpp>
#include <boost/rtl/priv/traits.hpp>
#include <boost/rtl/rtl.hpp>

#include <boost/iterator/reverse_iterator.hpp>


// rtl = Range Template Library
namespace boost { namespace rtl {

namespace detail {

    template< class r> 
    struct reverse_iterator_finder {
        typedef typename ::boost::rtl::range_finder<r>::range_type r_type;
        typedef typename r_type::iterator i_type;

        typedef ::boost::reverse_iterator<i_type> reverse_type;
    };
}


template< class r > 
struct reversed_range : public irange< typename ::boost::rtl::detail::reverse_iterator_finder<r>::reverse_type > {
    typedef irange< typename ::boost::rtl::detail::reverse_iterator_finder<r>::reverse_type > base;
    typedef typename ::boost::rtl::detail::reverse_iterator_finder<r>::i_type old_iterator;
    typedef typename ::boost::rtl::detail::reverse_iterator_finder<r>::reverse_type new_iterator;

    reversed_range( old_iterator first, old_iterator last)
        : base( new_iterator(last), new_iterator(first) ) {
    }

    reversed_range( r & rng) 
        : base( new_iterator( rng.end() ), 
                new_iterator( rng.begin() ) ) {
    }
};


#ifndef BOOST_RTL_WORKAROUND_VC6
template< class r> inline reversed_range<const r> 
reversed( const r & rng) {
    return reversed_range<const r>( rng.begin(), rng.end() );
}
#endif

template< class r> inline reversed_range<r> 
reversed( r & rng) {
    return reversed_range<r>( rng.begin(), rng.end() );
}



}}


#endif

