
// Boost. Range Template Library (rtl)
//
// Copyright 2003 John Torjo (john@torjo.com) and Matthew Wilson (matthew@synesis.com.au)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.


#ifndef BOOST_RTL_TRANSFORM_HPP_INCLUDED
#define BOOST_RTL_TRANSFORM_HPP_INCLUDED

#include <boost/rtl/priv/defs.hpp>
#include <boost/rtl/priv/traits.hpp>
#include <boost/rtl/rtl.hpp>

#include <boost/iterator/transform_iterator.hpp>
#include <functional>


// rtl = Range Template Library
namespace boost { namespace rtl {

namespace detail {

    template< class r, class function> 
    struct transform_iterator_finder {
        typedef typename ::boost::rtl::range_finder<r>::range_type r_type;
        typedef typename r_type::iterator i_type;

        typedef ::boost::transform_iterator<function, i_type> transform_type;
    };
}


template< class r, class function > 
struct transformed_range : public irange< typename ::boost::rtl::detail::transform_iterator_finder<r,function>::transform_type > {
    typedef irange< typename ::boost::rtl::detail::transform_iterator_finder<r,function>::transform_type> base;
    typedef typename ::boost::rtl::detail::transform_iterator_finder<r,function>::i_type old_iterator;
    typedef typename ::boost::rtl::detail::transform_iterator_finder<r,function>::transform_type new_iterator;

    transformed_range( old_iterator first, old_iterator last, function f)
        : base( new_iterator(first,f), new_iterator(last,f) ) {
    }

    transformed_range( r & rng, function f ) 
        : base( new_iterator( rng.begin(),f ), 
                new_iterator( rng.end(),f ) ) {
    }
};


#ifndef BOOST_RTL_WORKAROUND_VC6
template< class r, class function> inline transformed_range<const r, function> 
transformed( const r & rng, function f) {
    return transformed_range<const r, function>( rng.begin(), rng.end(), f);
}
template< class r, class arg, class res> inline transformed_range<const r, ::std::pointer_to_unary_function<arg,res> > 
transformed_f( const r & rng, res (*f)(arg) ) {
    return transformed_range<const r, ::std::pointer_to_unary_function<arg,res> >( rng.begin(), rng.end(), 
        ::std::pointer_to_unary_function<arg,res>(f) );
}
#endif


template< class r, class function> inline transformed_range<r, function> 
transformed( r & rng, function f) {
    return transformed_range<r, function>( rng.begin(), rng.end(), f);
}

// VC6 chokes if both these functions have the same name
template< class r, class arg, class res> inline transformed_range<r, ::std::pointer_to_unary_function<arg,res> > 
transformed_f( r & rng, res (*f)(arg) ) {
    return transformed_range<r, ::std::pointer_to_unary_function<arg,res> >( rng.begin(), rng.end(), 
        ::std::pointer_to_unary_function<arg,res>(f) );
}







}}


#endif
